; inherits: html

[
  (if_start)
  (else_if_start)
  (else_start)
  (if_end)
  (each_start)
  (each_end)
  (await_start)
  (then_start)
  (catch_start)
  (await_end)
  (key_start)
  (key_end)
  (snippet_start)
  (snippet_end)
  (expression)
  (html_tag)
  (const_tag)
  (debug_tag)
  (render_tag)
] @rainbow.scope

["{" "}"] @rainbow.bracket
