(escaped_percent_sign) @constant.character.escape

(flag) @constant.builtin

(text) @string

(width) @constant.numeric.integer
(precision) @constant.numeric.float

(conversion_specifier) @type

"." @punctuation.delimiter
"%" @punctuation.special
