[
  (configuration_uri_condition)
  (class_body)
  (block)
  (arguments)
  (list_literal)
  (formal_parameter_list)
  (optional_formal_parameters)
  (parameter_type_list)
  (record_type)
  (template_substitution)
  (index_selector)
  (if_statement)
  (assertion_arguments)
] @rainbow.scope

["(" ")" "[" "]" "{" "}"] @rainbow.bracket
